var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { asMaybeChallengeError } from 'edge-core-js';
import * as React from 'react';
import { ActivityIndicator, StyleSheet, View } from 'react-native';
import { AirshipModal } from 'react-native-airship';
import AntDesignIcon from 'react-native-vector-icons/AntDesign';
import { WebView } from 'react-native-webview';
import { lstrings } from '../../common/locales/strings';
import { useHandler } from '../../hooks/useHandler';
import { EdgeTouchableOpacity } from '../common/EdgeTouchableOpacity';
import { Airship } from '../services/AirshipInstance';
import { cacheStyles, useTheme } from '../services/ThemeContext';
import { EdgeText } from '../themed/EdgeText';
/**
 * Retries a task if the first attempt throws a challenge error,
 * and the user successfully solves the challenge.
 */
export function retryOnChallenge(opts) {
    return __awaiter(this, void 0, void 0, function* () {
        const { task, onCancel, onSuccess } = opts;
        return yield task().catch((error) => __awaiter(this, void 0, void 0, function* () {
            const challengeError = asMaybeChallengeError === null || asMaybeChallengeError === void 0 ? void 0 : asMaybeChallengeError(error);
            if (challengeError != null) {
                const result = yield Airship.show(bridge => (React.createElement(ChallengeModal, { bridge: bridge, challengeUri: challengeError.challengeUri })));
                if (result == null)
                    return onCancel();
                if (result) {
                    onSuccess === null || onSuccess === void 0 ? void 0 : onSuccess(challengeError.challengeId);
                    return yield task(challengeError.challengeId);
                }
            }
            throw error;
        }));
    });
}
export const ChallengeModal = (props) => {
    var _a, _b;
    const { bridge, challengeUri } = props;
    const theme = useTheme();
    const styles = getStyles(theme);
    const [loading, setLoading] = React.useState(true);
    const handleCancel = useHandler(() => bridge.resolve(undefined));
    const handleLoad = useHandler(() => setLoading(false));
    const handleLoading = useHandler((event) => {
        if (/\/success$/.test(event.url)) {
            bridge.resolve(true);
            return false;
        }
        if (/\/failure$/.test(event.url)) {
            bridge.resolve(false);
            return false;
        }
        return true;
    });
    const bgColor = (_a = serverColor(theme.modal)) !== null && _a !== void 0 ? _a : '1a1a1a';
    const fgColor = (_b = serverColor(theme.primaryText)) !== null && _b !== void 0 ? _b : 'fff';
    return (React.createElement(AirshipModal, { backgroundColor: theme.modal, bridge: bridge, 
        // Create a gap on top of the modal, so the user can tap to dismiss:
        margin: [theme.rem(5), 0, 0], padding: theme.rem(0.5), onCancel: handleCancel },
        React.createElement(View, { style: styles.titleContainer },
            React.createElement(EdgeText, { style: styles.titleText, numberOfLines: 2 }, lstrings.complete_captcha_title),
            React.createElement(EdgeTouchableOpacity, { style: styles.closeIconContainer, onPress: handleCancel },
                React.createElement(AntDesignIcon, { name: "close", color: theme.primaryText, size: theme.rem(1.25) }))),
        React.createElement(WebView, { javaScriptEnabled: true, source: {
                uri: challengeUri + `?bg=${bgColor}&fg=${fgColor}`
            }, style: styles.webview, onLoad: handleLoad, onShouldStartLoadWithRequest: handleLoading }),
        !loading ? null : (React.createElement(View, { pointerEvents: "box-none", style: styles.overlay },
            React.createElement(ActivityIndicator, { color: theme.primaryText, size: "large" })))));
};
const getStyles = cacheStyles((theme) => ({
    closeIconContainer: {
        margin: theme.rem(-0.5),
        padding: theme.rem(0.5)
    },
    titleContainer: {
        flexDirection: 'row',
        justifyContent: 'space-between',
        margin: theme.rem(0.5)
    },
    titleText: {
        color: theme.primaryText,
        flexShrink: 1,
        fontFamily: theme.fontFaceMedium,
        fontSize: theme.rem(1.2)
    },
    overlay: Object.assign(Object.assign({}, StyleSheet.absoluteFillObject), { alignItems: 'center', justifyContent: 'center' }),
    // eslint-disable-next-line react-native/no-color-literals
    webview: {
        alignSelf: 'stretch',
        // Allow the modal background to appear inside the WebView while loading.
        // This is a magic value from the WebView documentation,
        // so don't use the theme - normal colors don't do anything.
        backgroundColor: '#00000000',
        margin: theme.rem(0.5)
    }
}));
/**
 * Transforms a color into the server's query-string format.
 */
function serverColor(color) {
    const match = color.match(/^#([0-9A-Fa-f]{6})$/);
    if (match != null)
        return match[1].toLowerCase();
}
